//
// Software License for AMDiS
//
// Copyright (c) 2010 Dresden University of Technology 
// All rights reserved.
// Authors: Simon Vey, Thomas Witkowski et al.
//
// This file is part of AMDiS
//
// See also license.opensource.txt in the distribution.


#include "parallel/MeshPartitioner.h"
#include "Mesh.h"
#include "Traverse.h"
#include "Serializer.h"

namespace AMDiS {

  void MeshPartitioner::createInitialPartitioning() 
  {
    FUNCNAME("MeshPartitioner::createInitialPartitioning()");

    int mpiRank = mpiComm->Get_rank();
    int mpiSize = mpiComm->Get_size();
    int nLeaves = mesh->getNumberOfLeaves();
    int elPerRank = nLeaves / mpiSize;

    // === Create initial partitioning of the AMDiS mesh. ===

    elementInRank.clear();

    TraverseStack stack;
    ElInfo *elInfo = stack.traverseFirst(mesh, 0, Mesh::CALL_EL_LEVEL);
    while (elInfo) {
      Element *element = elInfo->getElement();

      if ((element->getIndex() >= mpiRank * elPerRank &&
	   element->getIndex() < (mpiRank + 1) * elPerRank) ||
	  (element->getIndex() >= mpiSize * elPerRank &&
	   mpiRank == mpiSize - 1))
	elementInRank[element->getIndex()] = true;
      else
	elementInRank[element->getIndex()] = false;
      
      elInfo = stack.traverseNext(elInfo);
    }
  }


  void MeshPartitioner::serialize(std::ostream &out)
  {
    SerUtil::serialize(out, elementInRank);
  }


  void MeshPartitioner::deserialize(std::istream &in)
  {
    SerUtil::deserialize(in, elementInRank);
  }

}
