// ============================================================================
// ==                                                                        ==
// == AMDiS - Adaptive multidimensional simulations                          ==
// ==                                                                        ==
// ==  http://www.amdis-fem.org                                              ==
// ==                                                                        ==
// ============================================================================
//
// Software License for AMDiS
//
// Copyright (c) 2010 Dresden University of Technology 
// All rights reserved.
// Authors: Simon Vey, Thomas Witkowski et al.
//
// This file is part of AMDiS
//
// See also license.opensource.txt in the distribution.



/** \file MeshPartitioner.h */

#ifndef AMDIS_MESH_PARTITIONER_H
#define AMDIS_MESH_PARTITIONER_H

#include <map>
#include "AMDiS_fwd.h"
#include "Mesh.h"

namespace AMDiS {


  using namespace std;


  enum PartitionMode {
    INITIAL = 0,          // initial partitioning of a unpartitioned mesh 
    ADAPTIVE_REPART = 1,  // repartitioning of a adaptively refined mesh
    REFINE_PART = 2       // quality improvement of the current partitioning
  };



  class MeshPartitioner
  {
  public:
    MeshPartitioner(MPI::Intracomm *comm)
      : mpiComm(comm),
	mesh(NULL),
	mapLocalGlobal(NULL)
    {}

    virtual ~MeshPartitioner() {}

    /// Creates an initial paritioning of the AMDiS mesh.
    void createInitialPartitioning();

    virtual bool partition(map<int, double> &elemWeights,
			   PartitionMode mode = INITIAL) = 0;

    /// Creates a map which stores for each element the rank that owns this element.
    virtual void getPartitionMap(map<int, int> &partitionMap) = 0;

    /// Write partitioner state to disk.
    void serialize(ostream &out);

    /// Read partitioner state from disk.
    void deserialize(istream &in);

    void setMesh(Mesh *m)
    {
      mesh = m;
    }

    void useLocalGlobalDofMap(map<DegreeOfFreedom, DegreeOfFreedom> *m)
    {
      mapLocalGlobal = m;
    }

    Mesh *getMesh()
    {
      return mesh;
    }

    map<int, bool>& getElementInRank()
    {
      return elementInRank;
    }

    map<int, vector<int> >& getRecvElements()
    {
      return recvElements;
    }

    map<int, vector<int> >& getSendElements()
    {
      return sendElements;
    }

  protected:
    MPI::Intracomm *mpiComm;

    Mesh *mesh;

    map<DegreeOfFreedom, DegreeOfFreedom> *mapLocalGlobal;

    /// Maps to each macro element index if it is in rank's partition or not.
    map<int, bool> elementInRank;

    map<int, vector<int> > recvElements, sendElements;
  };
}

#endif
